<?php

namespace Wappointment\Controllers;

use Wappointment\Models\Appointment as AppointmentModel;
use Wappointment\ClassConnect\Request;
use Wappointment\Services\Settings;
use Wappointment\Services\Appointment;
use Wappointment\Services\Service;
class AppointmentController extends \Wappointment\Controllers\RestController
{
    public function get(\Wappointment\ClassConnect\Request $request)
    {
        $appointment = \Wappointment\Models\Appointment::select(['start_at', 'status', 'end_at', 'type', 'client_id'])->where('status', '>=', \Wappointment\Models\Appointment::STATUS_AWAITING_CONFIRMATION)->where('edit_key', $request->input('appointmentkey'))->first();
        if (empty($appointment)) {
            throw new \WappointmentException("Can't find appointment", 1);
        }
        $appointmentData = $appointment->toArraySpecial();
        if (\Wappointment\Services\Settings::get('allow_rescheduling')) {
            $appointmentData['canRescheduleUntil'] = $appointment->canRescheduleUntilTimestamp();
        }
        if (\Wappointment\Services\Settings::get('allow_cancellation')) {
            $appointmentData['canCancelUntil'] = $appointment->canCancelUntilTimestamp();
        }
        return ['appointment' => $appointmentData, 'client' => $appointment->client()->select(['name', 'email', 'options'])->first(), 'service' => \Wappointment\Services\Service::get(), 'staff' => (new \Wappointment\WP\Staff($appointment->getStaffId()))->toArray(), 'date_format' => \Wappointment\Services\Settings::get('date_format'), 'time_format' => \Wappointment\Services\Settings::get('time_format'), 'date_time_union' => \Wappointment\Services\Settings::get('date_time_union', ' - ')];
    }
    public function cancel(\Wappointment\ClassConnect\Request $request)
    {
        $result = \Wappointment\Services\Appointment::tryCancel($request->input('appointmentkey'));
        if ($result) {
            return ['message' => 'Appointment has been canceled'];
        }
        return $result;
    }
}
