<?php

namespace Wappointment\Models;

use Wappointment\ClassConnect\Model;
use Wappointment\Services\Settings;
use Wappointment\ClassConnect\Carbon;
class Appointment extends \Wappointment\ClassConnect\Model
{
    protected $table = 'wappo_appointments';
    protected $fillable = ['start_at', 'end_at', 'edit_key', 'client_id', 'status', 'type', 'staff_id', 'service_id', 'options', 'location_id'];
    protected $casts = ['options' => 'array'];
    protected $dates = ['start_at', 'end_at'];
    const TYPE_PHYSICAL = 0;
    const TYPE_PHONE = 1;
    const TYPE_SKYPE = 2;
    /* const STATUS_ADMIN_CANCELLED = -2;
       const STATUS_CANCELLED = -1; */
    const STATUS_AWAITING_CONFIRMATION = 0;
    const STATUS_CONFIRMED = 1;
    public function getLocationSlug()
    {
        switch ($this->type) {
            case self::TYPE_PHYSICAL:
                return 'physical';
            case self::TYPE_PHONE:
                return 'phone';
            case self::TYPE_SKYPE:
                return 'skype';
        }
    }
    public function getStaffId()
    {
        return \Wappointment\Services\Settings::get('activeStaffId');
    }
    public function isPhone()
    {
        return self::TYPE_PHONE == $this->type;
    }
    public function isPhysical()
    {
        return self::TYPE_PHYSICAL == $this->type;
    }
    public function isSkype()
    {
        return self::TYPE_SKYPE == $this->type;
    }
    public static function getTypePhysical()
    {
        return self::TYPE_PHYSICAL;
    }
    public static function getTypePhone()
    {
        return self::TYPE_PHONE;
    }
    public static function getTypeSkype()
    {
        return self::TYPE_SKYPE;
    }
    public function toArraySpecial()
    {
        $array = parent::toArray();
        $array['start_at'] = $this->start_at->timestamp;
        $array['end_at'] = $this->end_at->timestamp;
        $array['type'] = $this->getLocationSlug();
        return $array;
    }
    public function getDurationInSec()
    {
        return $this->end_at->timestamp - $this->start_at->timestamp;
    }
    public function getDuration()
    {
        return $this->getDurationInSec() / 60 . 'min';
    }
    public function getStartsDayAndTime($timezone)
    {
        return $this->start_at->timezone($timezone)->format(\Wappointment\Services\Settings::get('date_format') . \Wappointment\Services\Settings::get('date_time_union') . \Wappointment\Services\Settings::get('time_format'));
    }
    private function getPageLink($view = 'reschedule-event')
    {
        static $page_link = '';
        if ($page_link == '') {
            $page_link = get_permalink(\Wappointment\Services\Settings::get('front_page'));
        }
        return $page_link . (\strpos($page_link, '?') !== false ? '&' : '?') . 'view=' . $view . '&appointmentkey=' . $this->edit_key;
    }
    public function getServiceName()
    {
        return $this->getService()->name;
    }
    public function getService()
    {
        static $services = [];
        if (empty($services[$this->service_id])) {
            $services[$this->service_id] = apply_filters('wappointment_get_appointment_service', \Wappointment\Services\Service::getObject(), $this->service_id);
        }
        return $services[$this->service_id];
    }
    public function getServiceAddress()
    {
        return $this->getService()->address;
    }
    public function getLinkAddEventToCalendar()
    {
        return $this->getPageLink('add-event-to-calendar');
    }
    public function getLinkRescheduleEvent()
    {
        return $this->getPageLink();
    }
    public function getLinkCancelEvent()
    {
        return $this->getPageLink('cancel-event');
    }
    public function getLinkNewEvent()
    {
        return $this->getPageLink('new-event');
    }
    public function client()
    {
        return $this->belongsTo(\Wappointment\Models\Client::class, 'client_id');
    }
    public function canRescheduleUntilTimestamp()
    {
        return $this->start_at->getTimestamp() - (int) \Wappointment\Services\Settings::get('hours_before_rescheduling_allowed') * 60 * 60;
    }
    public function canCancelUntilTimestamp()
    {
        return $this->start_at->getTimestamp() - (int) \Wappointment\Services\Settings::get('hours_before_cancellation_allowed') * 60 * 60;
    }
    public function canStillReschedule()
    {
        return $this->canRescheduleUntilTimestamp() - \Wappointment\ClassConnect\Carbon::now()->timestamp > 0;
    }
    public function canStillCancel()
    {
        return $this->status !== self::STATUS_CONFIRMED || $this->canCancelUntilTimestamp() - \Wappointment\ClassConnect\Carbon::now()->timestamp > 0;
    }
}
