<?php

namespace Wappointment\Services;

use Wappointment\WP\Helpers as WPHelpers;
use Wappointment\ClassConnect\Carbon;
use Wappointment\Models\WPUser;
use Wappointment\Models\WPUserMeta;
class ViewsData
{
    public function load($key)
    {
        if (\method_exists($this, $key)) {
            return $this->{$key}();
        }
        return apply_filters('wappointment_viewdata_' . $key, []);
    }
    private function regav()
    {
        $gravatar_img = get_avatar_url(\Wappointment\Services\Settings::get('activeStaffId'), ['size' => 40]);
        return apply_filters('wappointment_back_regav', ['regav' => \Wappointment\Services\Settings::getStaff('regav'), 'staffs' => \Wappointment\Models\WPUser::whereIn('ID', \Wappointment\Models\WPUserMeta::getUserIdWithRoles())->get(), 'activeStaffId' => \Wappointment\Services\Settings::get('activeStaffId'), 'activeStaffAvatar' => \Wappointment\Services\Settings::getStaff('avatarId') ? wp_get_attachment_image_src(\Wappointment\Services\Settings::getStaff('avatarId'))[0] : $gravatar_img, 'activeStaffGravatar' => $gravatar_img, 'activeStaffAvatarId' => \Wappointment\Services\Settings::getStaff('avatarId'), 'timezone' => \Wappointment\Services\Settings::getStaff('timezone'), 'timezones_list' => \Wappointment\Services\DateTime::tz(), 'savedTimezone' => \Wappointment\Services\Settings::hasStaff('timezone')]);
    }
    private function serverinfo()
    {
        return ['server' => \Wappointment\Services\Server::data()];
    }
    private function calsync()
    {
        return ['calurl' => \Wappointment\Services\Settings::getStaff('calurl')];
    }
    private function service()
    {
        return ['service' => \Wappointment\Services\Service::get()];
    }
    private function widget()
    {
        return apply_filters('wappointment_back_widget_editor', ['front_availability' => $this->front_availability(), 'widget' => (new \Wappointment\Services\WidgetSettings())->get(), 'widgetDefault' => (new \Wappointment\Services\WidgetSettings())->defaultSettings(), 'config' => ['service' => \Wappointment\Services\Service::get(), 'approval_mode' => \Wappointment\Services\Settings::get('approval_mode')], 'bgcolor' => \Wappointment\WP\Helpers::getThemeBgColor(), 'more' => get_theme_mods(), 'widgetFields' => (new \Wappointment\Services\WidgetSettings())->adminFieldsInfo()]);
    }
    private function widgetcancel()
    {
        return ['widget' => (new \Wappointment\Services\WidgetSettings())->get(), 'widgetDefault' => (new \Wappointment\Services\WidgetSettings())->defaultSettings(), 'staff' => (new \Wappointment\WP\Staff())->toArray()];
    }
    private function calendar()
    {
        $staff_timezone = \Wappointment\Services\Settings::getStaff('timezone');
        return apply_filters('wappointment_back_calendar', ['regav' => \Wappointment\Services\Settings::getStaff('regav'), 'availability' => \Wappointment\WP\Helpers::getStaffOption('availability'), 'timezone' => $staff_timezone, 'week_starts_on' => \Wappointment\Services\Settings::get('week_starts_on'), 'wizard_step' => \Wappointment\WP\Helpers::getOption('wizard_step'), 'timezones_list' => \Wappointment\Services\DateTime::tz(), 'now' => (new \Wappointment\ClassConnect\Carbon())->setTimezone($staff_timezone)->format('Y-m-d\\TH:i:00'), 'service' => \Wappointment\Services\Service::get(), 'date_format' => \Wappointment\Services\Settings::get('date_format'), 'time_format' => \Wappointment\Services\Settings::get('time_format'), 'date_time_union' => \Wappointment\Services\Settings::get('date_time_union', ' - '), 'preferredCountries' => \Wappointment\Services\Service::getObject()->getCountries(), 'widgetOptions' => (new \Wappointment\Services\WidgetSettings())->get()]);
    }
    private function settingsgeneral()
    {
        $service = \Wappointment\Services\Service::get();
        $widget = (new \Wappointment\Services\WidgetSettings())->get();
        return [
            // general
            'approval_mode' => \Wappointment\Services\Settings::get('approval_mode'),
            'date_format' => \Wappointment\Services\Settings::get('date_format'),
            'time_format' => \Wappointment\Services\Settings::get('time_format'),
            'date_time_union' => \Wappointment\Services\Settings::get('date_time_union', ' - '),
            'allow_cancellation' => \Wappointment\Services\Settings::get('allow_cancellation'),
            'allow_rescheduling' => \Wappointment\Services\Settings::get('allow_rescheduling'),
            'week_starts_on' => \Wappointment\Services\Settings::get('week_starts_on'),
            'hours_before_booking_allowed' => \Wappointment\Services\Settings::get('hours_before_booking_allowed'),
            'is_availability_set' => empty(\Wappointment\WP\Helpers::getStaffOption('availability')) ? false : true,
            'is_service_set' => empty($service['type']) && empty($service['name']) ? false : true,
            'is_widget_set' => empty($widget['form']) ? false : true,
            'hours_before_cancellation_allowed' => \Wappointment\Services\Settings::get('hours_before_cancellation_allowed'),
            'hours_before_rescheduling_allowed' => \Wappointment\Services\Settings::get('hours_before_rescheduling_allowed'),
            'timezone' => \Wappointment\Services\Settings::getStaff('timezone'),
            'widget' => (new \Wappointment\Services\WidgetSettings())->get(),
            'config' => ['service' => \Wappointment\Services\Service::get(), 'approval_mode' => \Wappointment\Services\Settings::get('approval_mode')],
            'bgcolor' => \Wappointment\WP\Helpers::getThemeBgColor(),
        ];
    }
    private function settingsnotifications()
    {
        return ['weekly_summary' => \Wappointment\Services\Settings::get('weekly_summary'), 'weekly_summary_day' => \Wappointment\Services\Settings::get('weekly_summary_day'), 'weekly_summary_time' => \Wappointment\Services\Settings::get('weekly_summary_time'), 'daily_summary' => \Wappointment\Services\Settings::get('daily_summary'), 'daily_summary_time' => \Wappointment\Services\Settings::get('daily_summary_time'), 'notify_new_appointments' => \Wappointment\Services\Settings::get('notify_new_appointments'), 'notify_canceled_appointments' => \Wappointment\Services\Settings::get('notify_canceled_appointments'), 'notify_rescheduled_appointments' => \Wappointment\Services\Settings::get('notify_rescheduled_appointments'), 'email_notifications' => \Wappointment\Services\Settings::get('email_notifications'), 'mail_status' => (bool) \Wappointment\Services\Settings::get('mail_status'), 'timezone' => \Wappointment\Services\Settings::getStaff('timezone'), 'time_format' => \Wappointment\Services\Settings::get('time_format')];
    }
    private function settingssync()
    {
        return ['is_calendar_sync_set' => empty(\Wappointment\Services\Settings::getStaff('calurl')) ? false : true, 'timezone' => \Wappointment\Services\Settings::getStaff('timezone'), 'date_format' => \Wappointment\Services\Settings::get('date_format'), 'time_format' => \Wappointment\Services\Settings::get('time_format'), 'wappointment_allowed' => \Wappointment\Services\Settings::get('wappointment_allowed'), 'date_time_union' => \Wappointment\Services\Settings::get('date_time_union', ' - '), 'last_checked' => \Wappointment\WP\Helpers::getStaffOption('last-calendar-checked'), 'last_parsed' => \Wappointment\WP\Helpers::getStaffOption('last-calendar-parsed'), 'last_process' => \Wappointment\WP\Helpers::getStaffOption('last-calendar-process')];
    }
    private function wizardinit()
    {
        return ['admin_email' => wp_get_current_user()->user_email, 'admin_name' => wp_get_current_user()->display_name];
    }
    private function settingsmailer()
    {
        return ['mail_config' => \Wappointment\Services\Settings::get('mail_config'), 'recipient' => wp_get_current_user()->user_email];
    }
    private function TESTprocessAvail($avails)
    {
        foreach ($avails as &$avail) {
            $avail[0] = \Wappointment\ClassConnect\Carbon::createFromTimestamp($avail[0])->setTimezone($this->timezone)->format('Y-m-d\\TH:i:00 T');
            $avail[1] = \Wappointment\ClassConnect\Carbon::createFromTimestamp($avail[1])->setTimezone($this->timezone)->format('Y-m-d\\TH:i:00 T');
        }
        return $avails;
    }
    private function all_versions_changes()
    {
        return ['versions' => \Wappointment\System\Status::allUpdates()];
    }
    private function front_availability()
    {
        $staff_availability = [];
        $staffs = [];
        foreach (\Wappointment\Services\Staff::getIds() as $staff_id) {
            $staff = new \Wappointment\WP\Staff($staff_id);
            $staff_availability[$staff_id] = $staff->getAvailability();
            $staffs[] = $staff->toArray();
        }
        return apply_filters('wappointment_front_availability', ['staffs' => $staffs, 'availability' => $staff_availability, 'timezones_list' => \Wappointment\Services\DateTime::tz(), 'week_starts_on' => \Wappointment\Services\Settings::get('week_starts_on'), 'date_format' => \Wappointment\Services\Settings::get('date_format'), 'time_format' => \Wappointment\Services\Settings::get('time_format'), 'min_bookable' => \Wappointment\Services\Settings::get('hours_before_booking_allowed'), 'date_time_union' => \Wappointment\Services\Settings::get('date_time_union', ' - '), 'now' => (new \Wappointment\ClassConnect\Carbon())->format('Y-m-d\\TH:i:00'), 'services' => \Wappointment\Services\Service::all()]);
    }
}
